-- -------------------------------------------------------------------------- --
-- HelmNCloak by kunda                                                        --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Enhances the Equipment Manager to let you control the helm/cloak           --
-- visibility for each set.                                                   --
--                                                                            --
-- Usage:                                                                     --
-- Just click on the icons in the Equipment Manager to change the helm/cloak  --
-- visibility for that set. This helm/cloak setting is used when you equip a  --
-- set.                                                                       --
--                                                                            --
-- Style matters!                                                             --
-- -------------------------------------------------------------------------- --

--  ------------------------------------------------------------------------  --
--                                    STOP                                    --
--                      Equipment Manager Paranoia Check                      --
                   if not GetNumEquipmentSets then return end                   
--  ------------------------------------------------------------------------  --

-- ---------------------------------------------------------------------------------------------------------------------
HelmNCloak = CreateFrame("Frame") -- container
HelmNCloak_Data = {}              -- SavedVariable options table

local iconhelm_show  = "Interface\\AddOns\\HelmNCloak\\iconhelm_show.tga"
local iconhelm_hide  = "Interface\\AddOns\\HelmNCloak\\iconhelm_hide.tga"
local iconcloak_show = "Interface\\AddOns\\HelmNCloak\\iconcloak_show.tga"
local iconcloak_hide = "Interface\\AddOns\\HelmNCloak\\iconcloak_hide.tga"
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function ChangeSetDisplay(which, setID)
	local setName
	local setTexture
	for i = 1, GetNumEquipmentSets() do
		setName, setTexture = GetEquipmentSetInfo(i)
		if setName and setTexture then
			if i == setID then
				break
			end
		end
	end

	if which == "helm" then
		if HelmNCloak_Data[setName].helm then
			_G["HelmNCloak_Frame"..setID.."HelmButtonTexture"]:SetTexture(iconhelm_hide)
		else
			_G["HelmNCloak_Frame"..setID.."HelmButtonTexture"]:SetTexture(iconhelm_show)
		end
		HelmNCloak_Data[setName].helm = not HelmNCloak_Data[setName].helm
	else
		if HelmNCloak_Data[setName].cloak then
			_G["HelmNCloak_Frame"..setID.."CloakButtonTexture"]:SetTexture(iconcloak_hide)
		else
			_G["HelmNCloak_Frame"..setID.."CloakButtonTexture"]:SetTexture(iconcloak_show)
		end
		HelmNCloak_Data[setName].cloak = not HelmNCloak_Data[setName].cloak
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function SetupNewLayout()
	GearManagerDialog:SetWidth(345) -- 261+(5*16)+4
	for i = 1, MAX_EQUIPMENT_SETS_PER_PLAYER do
		if i == 1 then
			_G["GearSetButton"..i]:SetPoint("TOPLEFT", GearManagerDialog, "TOPLEFT", 16, -32)
		elseif mod(i, NUM_GEARSETS_PER_ROW) == 1 then
			_G["GearSetButton"..i]:SetPoint("TOP", "GearSetButton"..(i-NUM_GEARSETS_PER_ROW), "BOTTOM", 0, -10)
		else
			_G["GearSetButton"..i]:SetPoint("LEFT", "GearSetButton"..(i-1), "RIGHT", 29, 0)
		end

		local frame = CreateFrame("Frame", "HelmNCloak_Frame"..i, _G["GearSetButton"..i])
		frame:ClearAllPoints()
		frame:SetPoint("TOPLEFT", "GearSetButton"..i, "TOPRIGHT", 0, 3)
		frame:SetWidth(22)
		frame:SetHeight(45)
		frame:Hide()
		local texture = frame:CreateTexture(nil, "BACKGROUND")
		texture:SetAllPoints(frame)
		texture:SetTexture("Interface\\Buttons\\UI-EmptySlot-Disabled")
		texture:SetVertexColor(1.0, 1.0, 1.0, 0.6)
		texture:SetTexCoord(32/64, 54/64, 9/64, 54/64)

		local button = CreateFrame("Button", "HelmNCloak_Frame"..i.."HelmButton", _G["HelmNCloak_Frame"..i])
		button:ClearAllPoints()
		button:SetPoint("TOPLEFT", "HelmNCloak_Frame"..i, "TOPLEFT", 2, -3)
		button:SetWidth(16)
		button:SetHeight(16)
		button:SetScript("OnClick", function() ChangeSetDisplay("helm", i) end)
		local normalTexture = button:CreateTexture("HelmNCloak_Frame"..i.."HelmButtonTexture", "BACKGROUND")
		normalTexture:SetAllPoints(button)
		normalTexture:SetTexture(iconhelm_show)
		button:SetNormalTexture(normalTexture)
		local highlightTexture = button:CreateTexture(nil, "BACKGROUND")
		highlightTexture:SetAllPoints(button)
		highlightTexture:SetTexture("Interface\\Buttons\\UI-Common-MouseHilight")
		highlightTexture:SetVertexColor(1.0, 1.0, 1.0, 0.4)
		highlightTexture:SetTexCoord(5/32, 27/32, 5/32, 27/32)
		button:SetHighlightTexture(highlightTexture)

		local button = CreateFrame("Button", "HelmNCloak_Frame"..i.."CloakButton", _G["HelmNCloak_Frame"..i])
		button:ClearAllPoints()
		button:SetPoint("TOPLEFT", "HelmNCloak_Frame"..i, "TOPLEFT", 2, -21)
		button:SetWidth(16)
		button:SetHeight(16)
		button:SetScript("OnClick", function() ChangeSetDisplay("cloak", i) end)
		local normalTexture = button:CreateTexture("HelmNCloak_Frame"..i.."CloakButtonTexture", "BACKGROUND")
		normalTexture:SetAllPoints(button)
		normalTexture:SetTexture(iconcloak_show)
		button:SetNormalTexture(normalTexture)
		local highlightTexture = button:CreateTexture(nil, "BACKGROUND")
		highlightTexture:SetAllPoints(button)
		highlightTexture:SetTexture("Interface\\Buttons\\UI-Common-MouseHilight")
		highlightTexture:SetVertexColor(1.0, 1.0, 1.0, 0.4)
		highlightTexture:SetTexCoord(5/32, 27/32, 5/32, 27/32)
		button:SetHighlightTexture(highlightTexture)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function UpdateHelmNCloak()
	local numSets = GetNumEquipmentSets()
	local IsHelm  = false
	local IsCloak = false
	if ShowingHelm()  then IsHelm  = true end
	if ShowingCloak() then IsCloak = true end

	for i = 1, numSets do
		local setName, setTexture = GetEquipmentSetInfo(i)
		if setName and setTexture then

			if not HelmNCloak_Data[setName] then
				HelmNCloak_Data[setName] = {}
				HelmNCloak_Data[setName].helm = IsHelm
				HelmNCloak_Data[setName].cloak = IsCloak
			end

			_G["HelmNCloak_Frame"..i]:Show()

			if HelmNCloak_Data[setName].helm then
				_G["HelmNCloak_Frame"..i.."HelmButtonTexture"]:SetTexture(iconhelm_show)
			else
				_G["HelmNCloak_Frame"..i.."HelmButtonTexture"]:SetTexture(iconhelm_hide)
			end

			if HelmNCloak_Data[setName].cloak then
				_G["HelmNCloak_Frame"..i.."CloakButtonTexture"]:SetTexture(iconcloak_show)
			else
				_G["HelmNCloak_Frame"..i.."CloakButtonTexture"]:SetTexture(iconcloak_hide)
			end

		end
	end

	for i = numSets + 1, MAX_EQUIPMENT_SETS_PER_PLAYER do
		_G["HelmNCloak_Frame"..i]:Hide()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function DeleteSet(setName)
	HelmNCloak_Data[setName] = nil
	UpdateHelmNCloak()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function SaveSet()
	UpdateHelmNCloak()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function SwitchSet(setName)
	if not setName then return end
	if not HelmNCloak_Data[setName].helm  then return end
	if not HelmNCloak_Data[setName].cloak then return end

	if EquipmentSetContainsLockedItems(setName) or UnitCastingInfo("player") then
		return
	end	
	
	local IsHelm  = false
	local IsCloak = false
	if ShowingHelm()  then IsHelm  = true end
	if ShowingCloak() then IsCloak = true end
	if HelmNCloak_Data[setName].helm  ~= IsHelm  then ShowHelm(HelmNCloak_Data[setName].helm)   end
	if HelmNCloak_Data[setName].cloak ~= IsCloak then ShowCloak(HelmNCloak_Data[setName].cloak) end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event)
	if event == "PLAYER_ENTERING_WORLD" then
		SetupNewLayout()
		UpdateHelmNCloak()
		hooksecurefunc("DeleteEquipmentSet", DeleteSet)
		hooksecurefunc("SaveEquipmentSet", SaveSet)
		hooksecurefunc("UseEquipmentSet", SwitchSet)
		HelmNCloak:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

HelmNCloak:RegisterEvent("PLAYER_ENTERING_WORLD")
HelmNCloak:SetScript("OnEvent", OnEvent)